@@ -160,9 +160,9 @@ struct _GtkMenuButtonPrivate
   GtkMenuButtonShowMenuCallback func;
   gpointer user_data;
 
-  GtkArrowType arrow_type;
   GtkWidget *align_widget;
   gpointer arrow_widget;
+  GtkArrowType arrow_type;
 };
 
 enum
@@ -174,7 +174,7 @@ enum
   PROP_DIRECTION
 };
 
-G_DEFINE_TYPE(GtkMenuButton, gtk_menu_button, GTK_TYPE_TOGGLE_BUTTON)
+G_DEFINE_TYPE_WITH_PRIVATE (GtkMenuButton, gtk_menu_button, GTK_TYPE_TOGGLE_BUTTON)
 
 static void gtk_menu_button_dispose (GObject *object);
 
@@ -488,8 +488,6 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
   GtkContainerClass *container_class = GTK_CONTAINER_CLASS (klass);
   GtkToggleButtonClass *toggle_button_class = GTK_TOGGLE_BUTTON_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (GtkMenuButtonPrivate));
-
   gobject_class->set_property = gtk_menu_button_set_property;
   gobject_class->get_property = gtk_menu_button_get_property;
   gobject_class->dispose = gtk_menu_button_dispose;
@@ -516,28 +514,13 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                                                         P_("The dropdown menu."),
                                                         GTK_TYPE_MENU,
                                                         G_PARAM_READWRITE));
-   /**
-   * GtkMenuButton:menu:
-   *
-   * The #GtkMenu that will be popped up when the button is clicked.
-   * This property has been renamed to "popup".  "menu" will be
-   * removed before 3.6.0.
-   *
-   * Since: 3.6
-   */
-  g_object_class_install_property (gobject_class,
-                                   PROP_POPUP, /* [sic] */
-                                   g_param_spec_object ("menu",
-                                                        P_("menu"),
-                                                        P_("The dropdown menu."),
-                                                        GTK_TYPE_MENU,
-                                                        G_PARAM_DEPRECATED | G_PARAM_READWRITE));
+
   /**
    * GtkMenuButton:menu-model:
    *
    * The #GMenuModel from which the menu to pop up will be created.
    * See gtk_menu_button_set_menu_model() for the interaction with the
-   * #GtkMenuButton:menu property.
+   * #GtkMenuButton:popup property.
    *
    * Since: 3.6
    */
@@ -596,7 +579,7 @@ gtk_menu_button_init (GtkMenuButton *menu_button)
 {
   GtkMenuButtonPrivate *priv;
 
-  priv = G_TYPE_INSTANCE_GET_PRIVATE (menu_button, GTK_TYPE_MENU_BUTTON, GtkMenuButtonPrivate);
+  priv = gtk_menu_button_get_instance_private (menu_button);
   menu_button->priv = priv;
   priv->arrow_type = GTK_ARROW_DOWN;
 
@@ -672,10 +655,7 @@ _gtk_menu_button_set_popup_with_func (GtkMenuButton                 *menu_button
     {
       if (gtk_widget_get_visible (GTK_WIDGET (priv->popup)))
         gtk_menu_shell_deactivate (GTK_MENU_SHELL (priv->popup));
-    }
 
-  if (priv->popup)
-    {
       g_signal_handlers_disconnect_by_func (priv->popup,
                                             menu_deactivate_cb,
                                             menu_button);
@@ -689,6 +669,7 @@ _gtk_menu_button_set_popup_with_func (GtkMenuButton                 *menu_button
       gtk_menu_attach_to_widget (GTK_MENU (priv->popup), GTK_WIDGET (menu_button),
                                  menu_detacher);
 
+      gtk_widget_set_visible (priv->popup, FALSE);
       gtk_widget_set_sensitive (GTK_WIDGET (menu_button), TRUE);
 
       g_signal_connect (priv->popup, "deactivate",
@@ -699,7 +680,7 @@ _gtk_menu_button_set_popup_with_func (GtkMenuButton                 *menu_button
       gtk_widget_set_sensitive (GTK_WIDGET (menu_button), FALSE);
     }
 
-  g_object_notify (G_OBJECT (menu_button), "menu");
+  g_object_notify (G_OBJECT (menu_button), "popup");
   g_object_notify (G_OBJECT (menu_button), "menu-model");
 }
 
@@ -752,13 +733,13 @@ gtk_menu_button_get_popup (GtkMenuButton *menu_button)
  * @menu_button: a #GtkMenuButton
  * @menu_model: (allow-none): a #GMenuModel
  *
- * Sets the #GMenuModel from which the #GtkMenuButton:menu property will be
+ * Sets the #GMenuModel from which the #GtkMenuButton:popup property will be
  * filled in, or %NULL to disable the button.
  *
  * The #GtkMenu will be created with gtk_menu_new_from_model(), so actions
  * will be connected as documented there.
  *
- * If you #GtkMenuButton:menu * is already set, then its content will be lost
+ * If #GtkMenuButton:popup is already set, then its content will be lost
  * and replaced by our newly created #GtkMenu.
  *
  * Since: 3.6
@@ -806,6 +787,23 @@ gtk_menu_button_get_menu_model (GtkMenuButton *menu_button)
   return menu_button->priv->model;
 }
 
+static void
+set_align_widget_pointer (GtkMenuButton *menu_button,
+                          GtkWidget     *align_widget)
+{
+  GtkMenuButtonPrivate *priv;
+
+  priv = menu_button->priv;
+
+  if (priv->align_widget)
+    g_object_remove_weak_pointer (G_OBJECT (priv->align_widget), (gpointer *) &priv->align_widget);
+
+  priv->align_widget = align_widget;
+
+  if (align_widget)
+    g_object_add_weak_pointer (G_OBJECT (priv->align_widget), (gpointer *) &priv->align_widget);
+}
+
 /**
  * gtk_menu_button_set_align_widget:
  * @menu_button: a #GtkMenuButton
@@ -832,10 +830,7 @@ gtk_menu_button_set_align_widget (GtkMenuButton *menu_button,
   if (priv->align_widget == align_widget)
     return;
 
-  priv->align_widget = align_widget;
-
-  if (priv->align_widget)
-    g_object_add_weak_pointer (G_OBJECT (priv->align_widget), (gpointer *) &priv->align_widget);
+  set_align_widget_pointer (menu_button, align_widget);
 
   g_object_notify (G_OBJECT (menu_button), "align-widget");
 }
@@ -870,8 +865,8 @@ gtk_menu_button_get_align_widget (GtkMenuButton *menu_button)
  * If the menu when popped out would have collided with screen edges,
  * we will do our best to keep it inside the screen and fully visible.
  *
- * If you pass GTK_ARROW_NONE for a @direction, the menu will behave
- * as if you passed GTK_ARROW_DOWN (although you won't see any arrows).
+ * If you pass %GTK_ARROW_NONE for a @direction, the menu will behave
+ * as if you passed %GTK_ARROW_DOWN (although you won't see any arrows).
  *
  * Since: 3.6
  */
@@ -926,8 +921,11 @@ gtk_menu_button_dispose (GObject *object)
                                             menu_deactivate_cb,
                                             object);
       gtk_menu_detach (GTK_MENU (priv->popup));
+      priv->popup = NULL;
     }
 
+  set_align_widget_pointer (GTK_MENU_BUTTON (object), NULL);
+
   g_clear_object (&priv->model);
 
   G_OBJECT_CLASS (gtk_menu_button_parent_class)->dispose (object);
