@@ -33,8 +33,6 @@
 #include "gtklabel.h"
 #include "gtkbox.h"
 #include "gtkimage.h"
-#include "gtkstock.h"
-#include "gtkiconfactory.h"
 #include "gtkintl.h"
 #include "gtkprivate.h"
 #include "gtktypebuiltins.h"
@@ -50,6 +48,10 @@
  * convenience widget; you could construct the equivalent of #GtkMessageDialog
  * from #GtkDialog without too much effort, but #GtkMessageDialog saves typing.
  *
+ * One difference from #GtkDialog is that #GtkMessageDialog sets the
+ * #GtkWindow:skip-taskbar-hint property to %TRUE, so that the dialog is hidden
+ * from the taskbar by default.
+ *
  * The easiest way to do a modal message dialog is to use gtk_dialog_run(), though
  * you can also pass in the %GTK_DIALOG_MODAL flag, gtk_dialog_run() automatically
  * makes the dialog modal and waits for the user to respond to it. gtk_dialog_run()
@@ -119,10 +121,6 @@ static void gtk_message_dialog_get_property (GObject          *object,
 static void gtk_message_dialog_add_buttons  (GtkMessageDialog *message_dialog,
 					     GtkButtonsType    buttons);
 static void      gtk_message_dialog_buildable_interface_init     (GtkBuildableIface *iface);
-static GObject * gtk_message_dialog_buildable_get_internal_child (GtkBuildable  *buildable,
-                                                                  GtkBuilder    *builder,
-                                                                  const gchar   *childname);
-
 
 enum {
   PROP_0,
@@ -137,6 +135,7 @@ enum {
 };
 
 G_DEFINE_TYPE_WITH_CODE (GtkMessageDialog, gtk_message_dialog, GTK_TYPE_DIALOG,
+                         G_ADD_PRIVATE (GtkMessageDialog)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE,
                                                 gtk_message_dialog_buildable_interface_init))
 
@@ -146,23 +145,10 @@ static void
 gtk_message_dialog_buildable_interface_init (GtkBuildableIface *iface)
 {
   parent_buildable_iface = g_type_interface_peek_parent (iface);
-  iface->get_internal_child = gtk_message_dialog_buildable_get_internal_child;
   iface->custom_tag_start = parent_buildable_iface->custom_tag_start;
   iface->custom_finished = parent_buildable_iface->custom_finished;
 }
 
-static GObject *
-gtk_message_dialog_buildable_get_internal_child (GtkBuildable *buildable,
-                                                 GtkBuilder   *builder,
-                                                 const gchar  *childname)
-{
-  if (strcmp (childname, "message_area") == 0)
-    return G_OBJECT (gtk_message_dialog_get_message_area (GTK_MESSAGE_DIALOG (buildable)));
-
-  return parent_buildable_iface->get_internal_child (buildable, builder, childname);
-}
-
-
 static void
 gtk_message_dialog_class_init (GtkMessageDialogClass *class)
 {
@@ -307,105 +293,59 @@ gtk_message_dialog_class_init (GtkMessageDialogClass *class)
 							GTK_TYPE_WIDGET,
 							GTK_PARAM_READABLE));
 
-  g_type_class_add_private (gobject_class, sizeof (GtkMessageDialogPrivate));
+  /* Setup Composite data */
+  gtk_widget_class_set_template_from_resource (widget_class, "/org/gtk/libgtk/gtkmessagedialog.ui");
+  gtk_widget_class_bind_template_child_private (widget_class, GtkMessageDialog, image);
+  gtk_widget_class_bind_template_child_private (widget_class, GtkMessageDialog, label);
+  gtk_widget_class_bind_template_child_private (widget_class, GtkMessageDialog, secondary_label);
+  gtk_widget_class_bind_template_child_internal_private (widget_class, GtkMessageDialog, message_area);
 }
 
 static void
 gtk_message_dialog_init (GtkMessageDialog *dialog)
 {
-  GtkWidget *hbox;
-  GtkDialog *message_dialog = GTK_DIALOG (dialog);
-  GtkWidget *action_area, *content_area;
   GtkMessageDialogPrivate *priv;
 
-  dialog->priv = G_TYPE_INSTANCE_GET_PRIVATE (dialog,
-                                              GTK_TYPE_MESSAGE_DIALOG,
-                                              GtkMessageDialogPrivate);
+  dialog->priv = gtk_message_dialog_get_instance_private (dialog);
   priv = dialog->priv;
 
-  content_area = gtk_dialog_get_content_area (message_dialog);
-  action_area = gtk_dialog_get_action_area (message_dialog);
-
-  gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-  gtk_window_set_title (GTK_WINDOW (dialog), "");
-  gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), TRUE);
-
   priv->has_primary_markup = FALSE;
   priv->has_secondary_text = FALSE;
-  priv->secondary_label = gtk_label_new (NULL);
-  gtk_widget_set_no_show_all (priv->secondary_label, TRUE);
-
-  priv->label = gtk_label_new (NULL);
-  priv->image = gtk_image_new_from_stock (NULL, GTK_ICON_SIZE_DIALOG);
-  gtk_widget_set_halign (priv->image, GTK_ALIGN_CENTER);
-  gtk_widget_set_valign (priv->image, GTK_ALIGN_START);
-
-  gtk_label_set_line_wrap  (GTK_LABEL (priv->label), TRUE);
-  gtk_label_set_selectable (GTK_LABEL (priv->label), TRUE);
-  gtk_widget_set_halign (priv->label, GTK_ALIGN_START);
-  gtk_widget_set_valign (priv->label, GTK_ALIGN_START);
-
-  gtk_label_set_line_wrap  (GTK_LABEL (priv->secondary_label), TRUE);
-  gtk_label_set_selectable (GTK_LABEL (priv->secondary_label), TRUE);
-  gtk_widget_set_halign (priv->secondary_label, GTK_ALIGN_START);
-  gtk_widget_set_valign (priv->secondary_label, GTK_ALIGN_START);
-
-  gtk_misc_set_alignment (GTK_MISC (priv->label), 0.0, 0.0);
-  gtk_misc_set_alignment (GTK_MISC (priv->secondary_label), 0.0, 0.0);
-
-  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
-  priv->message_area = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
-
-  gtk_box_pack_start (GTK_BOX (priv->message_area), priv->label,
-                      FALSE, FALSE, 0);
-
-  gtk_box_pack_start (GTK_BOX (priv->message_area), priv->secondary_label,
-                      TRUE, TRUE, 0);
-
-  gtk_box_pack_start (GTK_BOX (hbox), priv->image,
-                      FALSE, FALSE, 0);
-
-  gtk_box_pack_start (GTK_BOX (hbox), priv->message_area,
-                      TRUE, TRUE, 0);
-
-  gtk_box_pack_start (GTK_BOX (content_area),
-                      hbox,
-                      FALSE, FALSE, 0);
-
-  gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-  gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
-  gtk_box_set_spacing (GTK_BOX (content_area), 14); /* 14 + 2 * 5 = 24 */
-  gtk_container_set_border_width (GTK_CONTAINER (action_area), 5);
-  gtk_box_set_spacing (GTK_BOX (action_area), 6);
+  priv->has_primary_markup = FALSE;
+  priv->has_secondary_text = FALSE;
 
+  gtk_widget_init_template (GTK_WIDGET (dialog));
   gtk_message_dialog_style_updated (GTK_WIDGET (dialog));
-
-  gtk_widget_show_all (hbox);
 }
 
 static void
 setup_primary_label_font (GtkMessageDialog *dialog)
 {
   GtkMessageDialogPrivate *priv = dialog->priv;
-  gint size;
-  PangoFontDescription *font_desc;
-  GtkStyleContext *context;
-  GtkStateFlags state;
 
-  /* unset the font settings */
-  gtk_widget_override_font (priv->label, NULL);
-
-  if (priv->has_secondary_text && !priv->has_primary_markup)
+  if (!priv->has_primary_markup)
     {
-      context = gtk_widget_get_style_context (priv->label);
-      state = gtk_widget_get_state_flags (priv->label);
+      PangoAttrList *attributes;
+      PangoAttribute *attr;
 
-      size = pango_font_description_get_size (gtk_style_context_get_font (context, state));
-      font_desc = pango_font_description_new ();
-      pango_font_description_set_weight (font_desc, PANGO_WEIGHT_BOLD);
-      pango_font_description_set_size (font_desc, size * PANGO_SCALE_LARGE);
-      gtk_widget_override_font (priv->label, font_desc);
-      pango_font_description_free (font_desc);
+      attributes = pango_attr_list_new ();
+
+      attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
+      pango_attr_list_insert (attributes, attr);
+
+      if (priv->has_secondary_text)
+        {
+          attr = pango_attr_scale_new (PANGO_SCALE_LARGE);
+          pango_attr_list_insert (attributes, attr);
+        }
+
+      gtk_label_set_attributes (GTK_LABEL (priv->label), attributes);
+      pango_attr_list_unref (attributes);
+    }
+  else
+    {
+      /* unset the font settings */
+      gtk_label_set_attributes (GTK_LABEL (priv->label), NULL);
     }
 }
 
@@ -414,27 +354,32 @@ setup_type (GtkMessageDialog *dialog,
 	    GtkMessageType    type)
 {
   GtkMessageDialogPrivate *priv = dialog->priv;
-  const gchar *stock_id = NULL;
+  const gchar *name = NULL;
   AtkObject *atk_obj;
- 
+  GIcon *gicon = NULL;
+
   priv->message_type = type;
 
   switch (type)
     {
     case GTK_MESSAGE_INFO:
-      stock_id = GTK_STOCK_DIALOG_INFO;
+      name = _("Information");
+      gicon = g_themed_icon_new_with_default_fallbacks ("dialog-information-symbolic");
       break;
 
     case GTK_MESSAGE_QUESTION:
-      stock_id = GTK_STOCK_DIALOG_QUESTION;
+      name = _("Question");
+      gicon = g_themed_icon_new_with_default_fallbacks ("dialog-question-symbolic");
       break;
 
     case GTK_MESSAGE_WARNING:
-      stock_id = GTK_STOCK_DIALOG_WARNING;
+      name = _("Warning");
+      gicon = g_themed_icon_new_with_default_fallbacks ("dialog-warning-symbolic");
       break;
-      
+
     case GTK_MESSAGE_ERROR:
-      stock_id = GTK_STOCK_DIALOG_ERROR;
+      name = _("Error");
+      gicon = g_themed_icon_new_with_default_fallbacks ("dialog-error-symbolic");
       break;
 
     case GTK_MESSAGE_OTHER:
@@ -445,21 +390,16 @@ setup_type (GtkMessageDialog *dialog,
       break;
     }
 
-  if (stock_id)
-    gtk_image_set_from_stock (GTK_IMAGE (priv->image), stock_id,
-                              GTK_ICON_SIZE_DIALOG);
-      
+  gtk_image_set_from_gicon (GTK_IMAGE (priv->image), gicon, GTK_ICON_SIZE_DIALOG);
+  if (gicon)
+    g_object_unref (gicon);
+
   atk_obj = gtk_widget_get_accessible (GTK_WIDGET (dialog));
   if (GTK_IS_ACCESSIBLE (atk_obj))
     {
       atk_object_set_role (atk_obj, ATK_ROLE_ALERT);
-      if (stock_id)
-        {
-          GtkStockItem item;
-
-          gtk_stock_lookup (stock_id, &item);
-          atk_object_set_name (atk_obj, item.label);
-        }
+      if (name)
+        atk_object_set_name (atk_obj, name);
     }
 }
 
@@ -731,7 +671,7 @@ gtk_message_dialog_set_image (GtkMessageDialog *dialog,
 
   if (image == NULL)
     {
-      image = gtk_image_new_from_stock (NULL, GTK_ICON_SIZE_DIALOG);
+      image = gtk_image_new_from_icon_name (NULL, GTK_ICON_SIZE_DIALOG);
       gtk_widget_set_halign (image, GTK_ALIGN_CENTER);
       gtk_widget_set_valign (image, GTK_ALIGN_START);
     }
@@ -800,9 +740,6 @@ gtk_message_dialog_set_markup (GtkMessageDialog *message_dialog,
  * Sets the secondary text of the message dialog to be @message_format
  * (with printf()-style).
  *
- * Note that setting a secondary text makes the primary text become
- * bold, unless you have provided explicit markup.
- *
  * Since: 2.6
  */
 void
@@ -851,9 +788,6 @@ gtk_message_dialog_format_secondary_text (GtkMessageDialog *message_dialog,
  * printf()-style), which is marked up with the
  * <link linkend="PangoMarkupFormat">Pango text markup language</link>.
  *
- * Note that setting a secondary text makes the primary text become
- * bold, unless you have provided explicit markup.
- *
  * Due to an oversight, this function does not escape special XML characters
  * like gtk_message_dialog_new_with_markup() does. Thus, if the arguments
  * may contain special XML characters, you should use g_markup_printf_escaped()
@@ -942,28 +876,28 @@ gtk_message_dialog_add_buttons (GtkMessageDialog* message_dialog,
 
     case GTK_BUTTONS_OK:
       gtk_dialog_add_button (dialog,
-                             GTK_STOCK_OK,
+                             _("_OK"),
                              GTK_RESPONSE_OK);
       break;
 
     case GTK_BUTTONS_CLOSE:
       gtk_dialog_add_button (dialog,
-                             GTK_STOCK_CLOSE,
+                             _("_Close"),
                              GTK_RESPONSE_CLOSE);
       break;
 
     case GTK_BUTTONS_CANCEL:
       gtk_dialog_add_button (dialog,
-                             GTK_STOCK_CANCEL,
+                             _("_Cancel"),
                              GTK_RESPONSE_CANCEL);
       break;
 
     case GTK_BUTTONS_YES_NO:
       gtk_dialog_add_button (dialog,
-                             GTK_STOCK_NO,
+                             _("_No"),
                              GTK_RESPONSE_NO);
       gtk_dialog_add_button (dialog,
-                             GTK_STOCK_YES,
+                             _("_Yes"),
                              GTK_RESPONSE_YES);
       gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
 					       GTK_RESPONSE_YES,
@@ -973,10 +907,10 @@ gtk_message_dialog_add_buttons (GtkMessageDialog* message_dialog,
 
     case GTK_BUTTONS_OK_CANCEL:
       gtk_dialog_add_button (dialog,
-                             GTK_STOCK_CANCEL,
+                             _("_Cancel"),
                              GTK_RESPONSE_CANCEL);
       gtk_dialog_add_button (dialog,
-                             GTK_STOCK_OK,
+                             _("_OK"),
                              GTK_RESPONSE_OK);
       gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
 					       GTK_RESPONSE_OK,
@@ -1010,7 +944,5 @@ gtk_message_dialog_style_updated (GtkWidget *widget)
                                       MAX (0, border_width - 7));
     }
 
-  setup_primary_label_font (dialog);
-
   GTK_WIDGET_CLASS (gtk_message_dialog_parent_class)->style_updated (widget);
 }
