@@ -22,13 +22,13 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#ifndef __GTK_WIDGET_H__
+#define __GTK_WIDGET_H__
+
 #if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
 #error "Only <gtk/gtk.h> can be included directly."
 #endif
 
-#ifndef __GTK_WIDGET_H__
-#define __GTK_WIDGET_H__
-
 #include <gdk/gdk.h>
 #include <gtk/gtkaccelgroup.h>
 #include <gtk/gtkborder.h>
@@ -86,6 +86,23 @@ typedef 	GdkRectangle	   GtkAllocation;
 typedef void    (*GtkCallback)     (GtkWidget        *widget,
 				    gpointer          data);
 
+/**
+ * GtkTickCallback:
+ * @widget: the widget
+ * @frame_clock: the frame clock for the widget (same as calling gtk_widget_get_frame_clock())
+ * @user_data: user data passed to gtk_widget_add_tick_callback().
+ *
+ * Callback type for adding a function to update animations. See gtk_widget_add_tick_callback().
+ *
+ * Returns: %G_SOURCE_CONTINUE if the tick callback should continue to be called,
+ *  %G_SOURCE_REMOVE if the tick callback should be removed.
+ *
+ * Since: 3.8
+ */
+typedef gboolean (*GtkTickCallback) (GtkWidget     *widget,
+                                     GdkFrameClock *frame_clock,
+                                     gpointer       user_data);
+
 /**
  * GtkRequisition:
  * @width: the widget's desired width
@@ -416,15 +433,25 @@ struct _GtkWidgetClass
   gboolean     (* touch_event)            (GtkWidget     *widget,
                                            GdkEventTouch *event);
 
+  void         (* get_preferred_height_and_baseline_for_width)  (GtkWidget     *widget,
+								 gint           width,
+								 gint          *minimum_height,
+								 gint          *natural_height,
+								 gint          *minimum_baseline,
+								 gint          *natural_baseline);
+  void         (* adjust_baseline_request)(GtkWidget         *widget,
+                                           gint              *minimum_baseline,
+                                           gint              *natural_baseline);
+  void         (* adjust_baseline_allocation) (GtkWidget         *widget,
+					       gint              *baseline);
+  void         (*queue_draw_region)           (GtkWidget         *widget,
+					       const cairo_region_t *region);
+
   /*< private >*/
 
   GtkWidgetClassPrivate *priv;
 
   /* Padding for future expansion */
-  void (*_gtk_reserved2) (void);
-  void (*_gtk_reserved3) (void);
-  void (*_gtk_reserved4) (void);
-  void (*_gtk_reserved5) (void);
   void (*_gtk_reserved6) (void);
   void (*_gtk_reserved7) (void);
 };
@@ -440,60 +467,102 @@ struct _GtkWidgetAuxInfo
   GtkBorder margin;
 };
 
+GDK_AVAILABLE_IN_ALL
 GType	   gtk_widget_get_type		  (void) G_GNUC_CONST;
+GDK_AVAILABLE_IN_ALL
 GtkWidget* gtk_widget_new		  (GType		type,
 					   const gchar	       *first_property_name,
 					   ...);
+GDK_AVAILABLE_IN_ALL
 void	   gtk_widget_destroy		  (GtkWidget	       *widget);
+GDK_AVAILABLE_IN_ALL
 void	   gtk_widget_destroyed		  (GtkWidget	       *widget,
 					   GtkWidget	      **widget_pointer);
+GDK_AVAILABLE_IN_ALL
 void	   gtk_widget_unparent		  (GtkWidget	       *widget);
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_show                (GtkWidget           *widget);
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_hide                (GtkWidget           *widget);
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_show_now            (GtkWidget           *widget);
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_show_all            (GtkWidget           *widget);
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_set_no_show_all     (GtkWidget           *widget,
                                            gboolean             no_show_all);
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_widget_get_no_show_all     (GtkWidget           *widget);
+GDK_AVAILABLE_IN_ALL
 void	   gtk_widget_map		  (GtkWidget	       *widget);
+GDK_AVAILABLE_IN_ALL
 void	   gtk_widget_unmap		  (GtkWidget	       *widget);
+GDK_AVAILABLE_IN_ALL
 void	   gtk_widget_realize		  (GtkWidget	       *widget);
+GDK_AVAILABLE_IN_ALL
 void	   gtk_widget_unrealize		  (GtkWidget	       *widget);
 
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_draw                (GtkWidget           *widget,
                                            cairo_t             *cr);
 /* Queuing draws */
+GDK_AVAILABLE_IN_ALL
 void	   gtk_widget_queue_draw	  (GtkWidget	       *widget);
+GDK_AVAILABLE_IN_ALL
 void	   gtk_widget_queue_draw_area	  (GtkWidget	       *widget,
 					   gint                 x,
 					   gint                 y,
 					   gint                 width,
 					   gint                 height);
+GDK_AVAILABLE_IN_ALL
 void	   gtk_widget_queue_draw_region   (GtkWidget	       *widget,
                                            const cairo_region_t*region);
+GDK_AVAILABLE_IN_ALL
 void	   gtk_widget_queue_resize	  (GtkWidget	       *widget);
+GDK_AVAILABLE_IN_ALL
 void	   gtk_widget_queue_resize_no_redraw (GtkWidget *widget);
+GDK_AVAILABLE_IN_3_8
+GdkFrameClock* gtk_widget_get_frame_clock (GtkWidget           *widget);
+
 GDK_DEPRECATED_IN_3_0_FOR(gtk_widget_get_preferred_size)
 void       gtk_widget_size_request        (GtkWidget           *widget,
                                            GtkRequisition      *requisition);
+GDK_AVAILABLE_IN_ALL
 void	   gtk_widget_size_allocate	  (GtkWidget	       *widget,
 					   GtkAllocation       *allocation);
+GDK_AVAILABLE_IN_3_10
+void	   gtk_widget_size_allocate_with_baseline	  (GtkWidget	       *widget,
+							   GtkAllocation       *allocation,
+							   gint                 baseline);
 
+GDK_AVAILABLE_IN_ALL
 GtkSizeRequestMode  gtk_widget_get_request_mode               (GtkWidget      *widget);
+GDK_AVAILABLE_IN_ALL
 void                gtk_widget_get_preferred_width            (GtkWidget      *widget,
                                                                gint           *minimum_width,
                                                                gint           *natural_width);
+GDK_AVAILABLE_IN_ALL
 void                gtk_widget_get_preferred_height_for_width (GtkWidget      *widget,
                                                                gint            width,
                                                                gint           *minimum_height,
                                                                gint           *natural_height);
+GDK_AVAILABLE_IN_ALL
 void                gtk_widget_get_preferred_height           (GtkWidget      *widget,
                                                                gint           *minimum_height,
                                                                gint           *natural_height);
+GDK_AVAILABLE_IN_ALL
 void                gtk_widget_get_preferred_width_for_height (GtkWidget      *widget,
                                                                gint            height,
                                                                gint           *minimum_width,
                                                                gint           *natural_width);
+GDK_AVAILABLE_IN_3_10
+void   gtk_widget_get_preferred_height_and_baseline_for_width (GtkWidget     *widget,
+							       gint           width,
+							       gint          *minimum_height,
+							       gint          *natural_height,
+							       gint          *minimum_baseline,
+							       gint          *natural_baseline);
+GDK_AVAILABLE_IN_ALL
 void                gtk_widget_get_preferred_size             (GtkWidget      *widget,
                                                                GtkRequisition *minimum_size,
                                                                GtkRequisition *natural_size);
@@ -501,211 +570,329 @@ void                gtk_widget_get_preferred_size             (GtkWidget      *w
 GDK_DEPRECATED_IN_3_0_FOR(gtk_widget_get_preferred_size)
 void       gtk_widget_get_child_requisition (GtkWidget         *widget,
                                              GtkRequisition    *requisition);
+GDK_AVAILABLE_IN_ALL
 void	   gtk_widget_add_accelerator	  (GtkWidget           *widget,
 					   const gchar         *accel_signal,
 					   GtkAccelGroup       *accel_group,
 					   guint                accel_key,
 					   GdkModifierType      accel_mods,
 					   GtkAccelFlags        accel_flags);
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_widget_remove_accelerator  (GtkWidget           *widget,
 					   GtkAccelGroup       *accel_group,
 					   guint                accel_key,
 					   GdkModifierType      accel_mods);
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_set_accel_path      (GtkWidget           *widget,
 					   const gchar         *accel_path,
 					   GtkAccelGroup       *accel_group);
+GDK_AVAILABLE_IN_ALL
 GList*     gtk_widget_list_accel_closures (GtkWidget	       *widget);
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_widget_can_activate_accel  (GtkWidget           *widget,
                                            guint                signal_id);
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_widget_mnemonic_activate   (GtkWidget           *widget,
 					   gboolean             group_cycling);
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_widget_event		  (GtkWidget	       *widget,
 					   GdkEvent	       *event);
+GDK_AVAILABLE_IN_ALL
 gint       gtk_widget_send_expose         (GtkWidget           *widget,
 					   GdkEvent            *event);
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_widget_send_focus_change   (GtkWidget           *widget,
                                            GdkEvent            *event);
 
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_widget_activate		     (GtkWidget	       *widget);
      
+GDK_AVAILABLE_IN_ALL
 void	   gtk_widget_reparent		  (GtkWidget	       *widget,
 					   GtkWidget	       *new_parent);
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_widget_intersect		  (GtkWidget	       *widget,
 					   const GdkRectangle  *area,
 					   GdkRectangle	       *intersection);
+GDK_AVAILABLE_IN_ALL
 cairo_region_t *gtk_widget_region_intersect	  (GtkWidget	       *widget,
 					   const cairo_region_t     *region);
 
+GDK_AVAILABLE_IN_ALL
 void	gtk_widget_freeze_child_notify	  (GtkWidget	       *widget);
+GDK_AVAILABLE_IN_ALL
 void	gtk_widget_child_notify		  (GtkWidget	       *widget,
 					   const gchar	       *child_property);
+GDK_AVAILABLE_IN_ALL
 void	gtk_widget_thaw_child_notify	  (GtkWidget	       *widget);
 
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_set_can_focus       (GtkWidget           *widget,
                                            gboolean             can_focus);
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_widget_get_can_focus       (GtkWidget           *widget);
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_widget_has_focus           (GtkWidget           *widget);
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_widget_is_focus            (GtkWidget           *widget);
 GDK_AVAILABLE_IN_3_2
 gboolean   gtk_widget_has_visible_focus   (GtkWidget           *widget);
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_grab_focus          (GtkWidget           *widget);
 
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_set_can_default     (GtkWidget           *widget,
                                            gboolean             can_default);
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_widget_get_can_default     (GtkWidget           *widget);
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_widget_has_default         (GtkWidget           *widget);
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_grab_default        (GtkWidget           *widget);
 
+GDK_AVAILABLE_IN_ALL
 void      gtk_widget_set_receives_default (GtkWidget           *widget,
                                            gboolean             receives_default);
+GDK_AVAILABLE_IN_ALL
 gboolean  gtk_widget_get_receives_default (GtkWidget           *widget);
 
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_widget_has_grab            (GtkWidget           *widget);
 
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_widget_device_is_shadowed  (GtkWidget           *widget,
                                            GdkDevice           *device);
 
 
+GDK_AVAILABLE_IN_ALL
 void                  gtk_widget_set_name               (GtkWidget    *widget,
 							 const gchar  *name);
+GDK_AVAILABLE_IN_ALL
 const gchar *         gtk_widget_get_name               (GtkWidget    *widget);
 
+GDK_DEPRECATED_IN_3_0_FOR(gtk_widget_set_state_flags)
 void                  gtk_widget_set_state              (GtkWidget    *widget,
 							 GtkStateType  state);
+
+GDK_DEPRECATED_IN_3_0_FOR(gtk_widget_get_state_flags)
 GtkStateType          gtk_widget_get_state              (GtkWidget    *widget);
 
+GDK_AVAILABLE_IN_ALL
 void                  gtk_widget_set_state_flags        (GtkWidget     *widget,
                                                          GtkStateFlags  flags,
                                                          gboolean       clear);
+GDK_AVAILABLE_IN_ALL
 void                  gtk_widget_unset_state_flags      (GtkWidget     *widget,
                                                          GtkStateFlags  flags);
+GDK_AVAILABLE_IN_ALL
 GtkStateFlags         gtk_widget_get_state_flags        (GtkWidget     *widget);
 
+GDK_AVAILABLE_IN_ALL
 void                  gtk_widget_set_sensitive          (GtkWidget    *widget,
 							 gboolean      sensitive);
+GDK_AVAILABLE_IN_ALL
 gboolean              gtk_widget_get_sensitive          (GtkWidget    *widget);
+GDK_AVAILABLE_IN_ALL
 gboolean              gtk_widget_is_sensitive           (GtkWidget    *widget);
 
+GDK_AVAILABLE_IN_ALL
 void                  gtk_widget_set_visible            (GtkWidget    *widget,
                                                          gboolean      visible);
+GDK_AVAILABLE_IN_ALL
 gboolean              gtk_widget_get_visible            (GtkWidget    *widget);
+GDK_AVAILABLE_IN_ALL
+gboolean              gtk_widget_is_visible             (GtkWidget    *widget);
 
+GDK_AVAILABLE_IN_ALL
 void                  gtk_widget_set_has_window         (GtkWidget    *widget,
                                                          gboolean      has_window);
+GDK_AVAILABLE_IN_ALL
 gboolean              gtk_widget_get_has_window         (GtkWidget    *widget);
 
+GDK_AVAILABLE_IN_ALL
 gboolean              gtk_widget_is_toplevel            (GtkWidget    *widget);
+GDK_AVAILABLE_IN_ALL
 gboolean              gtk_widget_is_drawable            (GtkWidget    *widget);
+GDK_AVAILABLE_IN_ALL
 void                  gtk_widget_set_realized           (GtkWidget    *widget,
                                                          gboolean      realized);
+GDK_AVAILABLE_IN_ALL
 gboolean              gtk_widget_get_realized           (GtkWidget    *widget);
+GDK_AVAILABLE_IN_ALL
 void                  gtk_widget_set_mapped             (GtkWidget    *widget,
                                                          gboolean      mapped);
+GDK_AVAILABLE_IN_ALL
 gboolean              gtk_widget_get_mapped             (GtkWidget    *widget);
 
+GDK_AVAILABLE_IN_ALL
 void                  gtk_widget_set_app_paintable      (GtkWidget    *widget,
 							 gboolean      app_paintable);
+GDK_AVAILABLE_IN_ALL
 gboolean              gtk_widget_get_app_paintable      (GtkWidget    *widget);
 
+GDK_AVAILABLE_IN_ALL
 void                  gtk_widget_set_double_buffered    (GtkWidget    *widget,
 							 gboolean      double_buffered);
+GDK_AVAILABLE_IN_ALL
 gboolean              gtk_widget_get_double_buffered    (GtkWidget    *widget);
 
+GDK_AVAILABLE_IN_ALL
 void                  gtk_widget_set_redraw_on_allocate (GtkWidget    *widget,
 							 gboolean      redraw_on_allocate);
 
+GDK_AVAILABLE_IN_ALL
 void                  gtk_widget_set_parent             (GtkWidget    *widget,
 							 GtkWidget    *parent);
+GDK_AVAILABLE_IN_ALL
 GtkWidget           * gtk_widget_get_parent             (GtkWidget    *widget);
 
+GDK_AVAILABLE_IN_ALL
 void                  gtk_widget_set_parent_window      (GtkWidget    *widget,
 							 GdkWindow    *parent_window);
+GDK_AVAILABLE_IN_ALL
 GdkWindow           * gtk_widget_get_parent_window      (GtkWidget    *widget);
 
+GDK_AVAILABLE_IN_ALL
 void                  gtk_widget_set_child_visible      (GtkWidget    *widget,
 							 gboolean      is_visible);
+GDK_AVAILABLE_IN_ALL
 gboolean              gtk_widget_get_child_visible      (GtkWidget    *widget);
 
+GDK_AVAILABLE_IN_ALL
 void                  gtk_widget_set_window             (GtkWidget    *widget,
                                                          GdkWindow    *window);
+GDK_AVAILABLE_IN_ALL
 GdkWindow           * gtk_widget_get_window             (GtkWidget    *widget);
+GDK_AVAILABLE_IN_3_8
+void                  gtk_widget_register_window        (GtkWidget    *widget,
+                                                         GdkWindow    *window);
+GDK_AVAILABLE_IN_3_8
+void                  gtk_widget_unregister_window      (GtkWidget    *widget,
+                                                         GdkWindow    *window);
 
+GDK_AVAILABLE_IN_ALL
 int                   gtk_widget_get_allocated_width    (GtkWidget     *widget);
+GDK_AVAILABLE_IN_ALL
 int                   gtk_widget_get_allocated_height   (GtkWidget     *widget);
+GDK_AVAILABLE_IN_3_10
+int                   gtk_widget_get_allocated_baseline (GtkWidget     *widget);
 
+GDK_AVAILABLE_IN_ALL
 void                  gtk_widget_get_allocation         (GtkWidget     *widget,
                                                          GtkAllocation *allocation);
+GDK_AVAILABLE_IN_ALL
 void                  gtk_widget_set_allocation         (GtkWidget     *widget,
                                                          const GtkAllocation *allocation);
 
+GDK_DEPRECATED_IN_3_0_FOR(gtk_widget_get_preferred_width and gtk_widget_get_preferred_height)
 void                  gtk_widget_get_requisition        (GtkWidget     *widget,
                                                          GtkRequisition *requisition);
 
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_widget_child_focus         (GtkWidget           *widget,
                                            GtkDirectionType     direction);
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_widget_keynav_failed       (GtkWidget           *widget,
                                            GtkDirectionType     direction);
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_error_bell          (GtkWidget           *widget);
 
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_set_size_request    (GtkWidget           *widget,
                                            gint                 width,
                                            gint                 height);
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_get_size_request    (GtkWidget           *widget,
                                            gint                *width,
                                            gint                *height);
+GDK_AVAILABLE_IN_ALL
 void	   gtk_widget_set_events	  (GtkWidget	       *widget,
 					   gint			events);
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_add_events          (GtkWidget           *widget,
 					   gint	                events);
+GDK_AVAILABLE_IN_ALL
 void	   gtk_widget_set_device_events	  (GtkWidget	       *widget,
                                            GdkDevice           *device,
 					   GdkEventMask		events);
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_add_device_events   (GtkWidget           *widget,
                                            GdkDevice           *device,
 					   GdkEventMask         events);
+GDK_AVAILABLE_IN_3_8
+void	   gtk_widget_set_opacity	  (GtkWidget	       *widget,
+					   double		opacity);
+GDK_AVAILABLE_IN_3_8
+double	   gtk_widget_get_opacity	  (GtkWidget	       *widget);
 
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_set_device_enabled  (GtkWidget    *widget,
                                            GdkDevice    *device,
                                            gboolean      enabled);
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_widget_get_device_enabled  (GtkWidget    *widget,
                                            GdkDevice    *device);
 
+GDK_AVAILABLE_IN_ALL
 GtkWidget*   gtk_widget_get_toplevel	(GtkWidget	*widget);
+GDK_AVAILABLE_IN_ALL
 GtkWidget*   gtk_widget_get_ancestor	(GtkWidget	*widget,
 					 GType		 widget_type);
+GDK_AVAILABLE_IN_ALL
 GdkVisual*   gtk_widget_get_visual	(GtkWidget	*widget);
+GDK_AVAILABLE_IN_ALL
 void         gtk_widget_set_visual	(GtkWidget	*widget,
                                          GdkVisual      *visual);
 
+GDK_AVAILABLE_IN_ALL
 GdkScreen *   gtk_widget_get_screen      (GtkWidget *widget);
+GDK_AVAILABLE_IN_ALL
 gboolean      gtk_widget_has_screen      (GtkWidget *widget);
+GDK_AVAILABLE_IN_3_10
+gint          gtk_widget_get_scale_factor (GtkWidget *widget);
+GDK_AVAILABLE_IN_ALL
 GdkDisplay *  gtk_widget_get_display     (GtkWidget *widget);
+GDK_AVAILABLE_IN_ALL
 GdkWindow *   gtk_widget_get_root_window (GtkWidget *widget);
+GDK_AVAILABLE_IN_ALL
 GtkSettings*  gtk_widget_get_settings    (GtkWidget *widget);
+GDK_AVAILABLE_IN_ALL
 GtkClipboard *gtk_widget_get_clipboard   (GtkWidget *widget,
 					  GdkAtom    selection);
 
 
 /* Expand flags and related support */
+GDK_AVAILABLE_IN_ALL
 gboolean gtk_widget_get_hexpand          (GtkWidget      *widget);
+GDK_AVAILABLE_IN_ALL
 void     gtk_widget_set_hexpand          (GtkWidget      *widget,
                                           gboolean        expand);
+GDK_AVAILABLE_IN_ALL
 gboolean gtk_widget_get_hexpand_set      (GtkWidget      *widget);
+GDK_AVAILABLE_IN_ALL
 void     gtk_widget_set_hexpand_set      (GtkWidget      *widget,
                                           gboolean        set);
+GDK_AVAILABLE_IN_ALL
 gboolean gtk_widget_get_vexpand          (GtkWidget      *widget);
+GDK_AVAILABLE_IN_ALL
 void     gtk_widget_set_vexpand          (GtkWidget      *widget,
                                           gboolean        expand);
+GDK_AVAILABLE_IN_ALL
 gboolean gtk_widget_get_vexpand_set      (GtkWidget      *widget);
+GDK_AVAILABLE_IN_ALL
 void     gtk_widget_set_vexpand_set      (GtkWidget      *widget,
                                           gboolean        set);
+GDK_AVAILABLE_IN_ALL
 void     gtk_widget_queue_compute_expand (GtkWidget      *widget);
+GDK_AVAILABLE_IN_ALL
 gboolean gtk_widget_compute_expand       (GtkWidget      *widget,
                                           GtkOrientation  orientation);
 
 
 /* Multidevice support */
+GDK_AVAILABLE_IN_ALL
 gboolean         gtk_widget_get_support_multidevice (GtkWidget      *widget);
+GDK_AVAILABLE_IN_ALL
 void             gtk_widget_set_support_multidevice (GtkWidget      *widget,
                                                      gboolean        support_multidevice);
 
@@ -716,31 +903,48 @@ void             gtk_widget_class_set_accessible_type    (GtkWidgetClass     *wi
 GDK_AVAILABLE_IN_3_2
 void             gtk_widget_class_set_accessible_role    (GtkWidgetClass     *widget_class,
                                                           AtkRole             role);
+GDK_AVAILABLE_IN_ALL
 AtkObject*       gtk_widget_get_accessible               (GtkWidget          *widget);
 
 
 /* Margin and alignment */
+GDK_AVAILABLE_IN_ALL
 GtkAlign gtk_widget_get_halign        (GtkWidget *widget);
+GDK_AVAILABLE_IN_ALL
 void     gtk_widget_set_halign        (GtkWidget *widget,
                                        GtkAlign   align);
+GDK_AVAILABLE_IN_ALL
 GtkAlign gtk_widget_get_valign        (GtkWidget *widget);
+GDK_AVAILABLE_IN_3_10
+GtkAlign gtk_widget_get_valign_with_baseline (GtkWidget *widget);
+GDK_AVAILABLE_IN_ALL
 void     gtk_widget_set_valign        (GtkWidget *widget,
                                        GtkAlign   align);
+GDK_AVAILABLE_IN_ALL
 gint     gtk_widget_get_margin_left   (GtkWidget *widget);
+GDK_AVAILABLE_IN_ALL
 void     gtk_widget_set_margin_left   (GtkWidget *widget,
                                        gint       margin);
+GDK_AVAILABLE_IN_ALL
 gint     gtk_widget_get_margin_right  (GtkWidget *widget);
+GDK_AVAILABLE_IN_ALL
 void     gtk_widget_set_margin_right  (GtkWidget *widget,
                                        gint       margin);
+GDK_AVAILABLE_IN_ALL
 gint     gtk_widget_get_margin_top    (GtkWidget *widget);
+GDK_AVAILABLE_IN_ALL
 void     gtk_widget_set_margin_top    (GtkWidget *widget,
                                        gint       margin);
+GDK_AVAILABLE_IN_ALL
 gint     gtk_widget_get_margin_bottom (GtkWidget *widget);
+GDK_AVAILABLE_IN_ALL
 void     gtk_widget_set_margin_bottom (GtkWidget *widget,
                                        gint       margin);
 
 
+GDK_AVAILABLE_IN_ALL
 gint	     gtk_widget_get_events	(GtkWidget	*widget);
+GDK_AVAILABLE_IN_ALL
 GdkEventMask gtk_widget_get_device_events (GtkWidget	*widget,
                                            GdkDevice    *device);
 GDK_DEPRECATED_IN_3_4_FOR(gdk_window_get_device_position)
@@ -748,9 +952,11 @@ void	     gtk_widget_get_pointer	(GtkWidget	*widget,
 					 gint		*x,
 					 gint		*y);
 
+GDK_AVAILABLE_IN_ALL
 gboolean     gtk_widget_is_ancestor	(GtkWidget	*widget,
 					 GtkWidget	*ancestor);
 
+GDK_AVAILABLE_IN_ALL
 gboolean     gtk_widget_translate_coordinates (GtkWidget  *src_widget,
 					       GtkWidget  *dest_widget,
 					       gint        src_x,
@@ -760,33 +966,44 @@ gboolean     gtk_widget_translate_coordinates (GtkWidget  *src_widget,
 
 /* Hide widget and return TRUE.
  */
+GDK_AVAILABLE_IN_ALL
 gboolean     gtk_widget_hide_on_delete	(GtkWidget	*widget);
 
 /* Functions to override widget styling */
+GDK_AVAILABLE_IN_ALL
 void         gtk_widget_override_color            (GtkWidget     *widget,
                                                    GtkStateFlags  state,
                                                    const GdkRGBA *color);
+GDK_AVAILABLE_IN_ALL
 void         gtk_widget_override_background_color (GtkWidget     *widget,
                                                    GtkStateFlags  state,
                                                    const GdkRGBA *color);
 
+GDK_AVAILABLE_IN_ALL
 void         gtk_widget_override_font             (GtkWidget                  *widget,
                                                    const PangoFontDescription *font_desc);
 
+GDK_AVAILABLE_IN_ALL
 void         gtk_widget_override_symbolic_color   (GtkWidget     *widget,
                                                    const gchar   *name,
                                                    const GdkRGBA *color);
+GDK_AVAILABLE_IN_ALL
 void         gtk_widget_override_cursor           (GtkWidget       *widget,
                                                    const GdkRGBA   *cursor,
                                                    const GdkRGBA   *secondary_cursor);
 
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_reset_style       (GtkWidget      *widget);
 
+GDK_AVAILABLE_IN_ALL
 PangoContext *gtk_widget_create_pango_context (GtkWidget   *widget);
+GDK_AVAILABLE_IN_ALL
 PangoContext *gtk_widget_get_pango_context    (GtkWidget   *widget);
+GDK_AVAILABLE_IN_ALL
 PangoLayout  *gtk_widget_create_pango_layout  (GtkWidget   *widget,
 					       const gchar *text);
 
+GDK_DEPRECATED_IN_3_10_FOR(gtk_icon_theme_load_icon)
 GdkPixbuf    *gtk_widget_render_icon_pixbuf   (GtkWidget   *widget,
                                                const gchar *stock_id,
                                                GtkIconSize  size);
@@ -794,92 +1011,131 @@ GdkPixbuf    *gtk_widget_render_icon_pixbuf   (GtkWidget   *widget,
 /* handle composite names for GTK_COMPOSITE_CHILD widgets,
  * the returned name is newly allocated.
  */
+GDK_DEPRECATED_IN_3_10_FOR(gtk_widget_class_set_template)
 void   gtk_widget_set_composite_name	(GtkWidget	*widget,
 					 const gchar   	*name);
+GDK_DEPRECATED_IN_3_10_FOR(gtk_widget_class_set_template)
 gchar* gtk_widget_get_composite_name	(GtkWidget	*widget);
      
 /* Push/pop pairs, to change default values upon a widget's creation.
  * This will override the values that got set by the
  * gtk_widget_set_default_* () functions.
  */
+GDK_DEPRECATED_IN_3_10_FOR(gtk_widget_class_set_template)
 void	     gtk_widget_push_composite_child (void);
+GDK_DEPRECATED_IN_3_10_FOR(gtk_widget_class_set_template)
 void	     gtk_widget_pop_composite_child  (void);
 
 /* widget style properties
  */
+GDK_AVAILABLE_IN_ALL
 void gtk_widget_class_install_style_property        (GtkWidgetClass     *klass,
 						     GParamSpec         *pspec);
+GDK_AVAILABLE_IN_ALL
 void gtk_widget_class_install_style_property_parser (GtkWidgetClass     *klass,
 						     GParamSpec         *pspec,
 						     GtkRcPropertyParser parser);
+GDK_AVAILABLE_IN_ALL
 GParamSpec*  gtk_widget_class_find_style_property   (GtkWidgetClass     *klass,
 						     const gchar        *property_name);
+GDK_AVAILABLE_IN_ALL
 GParamSpec** gtk_widget_class_list_style_properties (GtkWidgetClass     *klass,
 						     guint              *n_properties);
+GDK_AVAILABLE_IN_ALL
 void gtk_widget_style_get_property (GtkWidget	     *widget,
 				    const gchar    *property_name,
 				    GValue	     *value);
+GDK_AVAILABLE_IN_ALL
 void gtk_widget_style_get_valist   (GtkWidget	     *widget,
 				    const gchar    *first_property_name,
 				    va_list         var_args);
+GDK_AVAILABLE_IN_ALL
 void gtk_widget_style_get          (GtkWidget	     *widget,
 				    const gchar    *first_property_name,
 				    ...) G_GNUC_NULL_TERMINATED;
 
 /* Functions for setting directionality for widgets */
 
+GDK_AVAILABLE_IN_ALL
 void             gtk_widget_set_direction         (GtkWidget        *widget,
 						   GtkTextDirection  dir);
+GDK_AVAILABLE_IN_ALL
 GtkTextDirection gtk_widget_get_direction         (GtkWidget        *widget);
 
+GDK_AVAILABLE_IN_ALL
 void             gtk_widget_set_default_direction (GtkTextDirection  dir);
+GDK_AVAILABLE_IN_ALL
 GtkTextDirection gtk_widget_get_default_direction (void);
 
 /* Compositing manager functionality */
+GDK_AVAILABLE_IN_ALL
 gboolean gtk_widget_is_composited (GtkWidget *widget);
 
 /* Counterpart to gdk_window_shape_combine_region.
  */
+GDK_AVAILABLE_IN_ALL
 void	     gtk_widget_shape_combine_region (GtkWidget *widget,
                                               cairo_region_t *region);
+GDK_AVAILABLE_IN_ALL
 void	     gtk_widget_input_shape_combine_region (GtkWidget *widget,
                                                     cairo_region_t *region);
 
+GDK_AVAILABLE_IN_ALL
 GList* gtk_widget_list_mnemonic_labels  (GtkWidget *widget);
+GDK_AVAILABLE_IN_ALL
 void   gtk_widget_add_mnemonic_label    (GtkWidget *widget,
 					 GtkWidget *label);
+GDK_AVAILABLE_IN_ALL
 void   gtk_widget_remove_mnemonic_label (GtkWidget *widget,
 					 GtkWidget *label);
 
+GDK_AVAILABLE_IN_ALL
 void                  gtk_widget_set_tooltip_window    (GtkWidget   *widget,
                                                         GtkWindow   *custom_window);
+GDK_AVAILABLE_IN_ALL
 GtkWindow *gtk_widget_get_tooltip_window    (GtkWidget   *widget);
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_trigger_tooltip_query (GtkWidget   *widget);
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_set_tooltip_text      (GtkWidget   *widget,
                                              const gchar *text);
+GDK_AVAILABLE_IN_ALL
 gchar *    gtk_widget_get_tooltip_text      (GtkWidget   *widget);
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_set_tooltip_markup    (GtkWidget   *widget,
                                              const gchar *markup);
+GDK_AVAILABLE_IN_ALL
 gchar *    gtk_widget_get_tooltip_markup    (GtkWidget   *widget);
+GDK_AVAILABLE_IN_ALL
 void       gtk_widget_set_has_tooltip       (GtkWidget   *widget,
 					     gboolean     has_tooltip);
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_widget_get_has_tooltip       (GtkWidget   *widget);
 
+GDK_AVAILABLE_IN_ALL
 gboolean   gtk_cairo_should_draw_window     (cairo_t     *cr,
                                              GdkWindow   *window);
+GDK_AVAILABLE_IN_ALL
 void       gtk_cairo_transform_to_window    (cairo_t     *cr,
                                              GtkWidget   *widget,
                                              GdkWindow   *window);
 
+GDK_AVAILABLE_IN_ALL
 GType           gtk_requisition_get_type (void) G_GNUC_CONST;
+GDK_AVAILABLE_IN_ALL
 GtkRequisition *gtk_requisition_new      (void) G_GNUC_MALLOC;
+GDK_AVAILABLE_IN_ALL
 GtkRequisition *gtk_requisition_copy     (const GtkRequisition *requisition);
+GDK_AVAILABLE_IN_ALL
 void            gtk_requisition_free     (GtkRequisition       *requisition);
 
+GDK_AVAILABLE_IN_ALL
 gboolean     gtk_widget_in_destruction (GtkWidget *widget);
 
+GDK_AVAILABLE_IN_ALL
 GtkStyleContext * gtk_widget_get_style_context (GtkWidget *widget);
 
+GDK_AVAILABLE_IN_ALL
 GtkWidgetPath *   gtk_widget_get_path (GtkWidget *widget);
 
 GDK_AVAILABLE_IN_3_4
@@ -891,6 +1147,155 @@ void                    gtk_widget_insert_action_group                  (GtkWidg
                                                                          const gchar  *name,
                                                                          GActionGroup *group);
 
+
+
+GDK_AVAILABLE_IN_3_8
+guint gtk_widget_add_tick_callback (GtkWidget       *widget,
+                                    GtkTickCallback  callback,
+                                    gpointer         user_data,
+                                    GDestroyNotify   notify);
+
+GDK_AVAILABLE_IN_3_8
+void gtk_widget_remove_tick_callback (GtkWidget       *widget,
+                                      guint            id);
+
+/**
+ * gtk_widget_class_bind_template_callback:
+ * @widget_class: a #GtkWidgetClass
+ * @callback: the callback symbol
+ *
+ * Binds a callback function defined in a template to the @widget_class.
+ *
+ * This macro is a convenience wrapper around the
+ * gtk_widget_class_bind_template_callback_full() function.
+ *
+ * Since: 3.10
+ */
+#define gtk_widget_class_bind_template_callback(widget_class, callback) \
+  gtk_widget_class_bind_template_callback_full (GTK_WIDGET_CLASS (widget_class), \
+                                                #callback, \
+                                                G_CALLBACK (callback))
+
+/**
+ * gtk_widget_class_bind_template_child:
+ * @widget_class: a #GtkWidgetClass
+ * @TypeName: the type name of this widget
+ * @member_name: name of the instance member in the instance struct for @data_type
+ *
+ * Binds a child widget defined in a template to the @widget_class.
+ *
+ * This macro is a convenience wrapper around the
+ * gtk_widget_class_bind_template_child_full() function.
+ *
+ * This macro will use the offset of the @member_name inside the @TypeName
+ * instance structure.
+ *
+ * Since: 3.10
+ */
+#define gtk_widget_class_bind_template_child(widget_class, TypeName, member_name) \
+  gtk_widget_class_bind_template_child_full (widget_class, \
+                                             #member_name, \
+                                             FALSE, \
+                                             G_STRUCT_OFFSET (TypeName, member_name))
+
+/**
+ * gtk_widget_class_bind_template_child_internal:
+ * @widget_class: a #GtkWidgetClass
+ * @TypeName: the type name, in CamelCase
+ * @member_name: name of the instance member in the instance struct for @data_type
+ *
+ * Binds a child widget defined in a template to the @widget_class, and
+ * also makes it available as an internal child in GtkBuilder, under the
+ * name @member_name.
+ *
+ * This macro is a convenience wrapper around the
+ * gtk_widget_class_bind_template_child_full() function.
+ *
+ * This macro will use the offset of the @member_name inside the @TypeName
+ * instance structure.
+ *
+ * Since: 3.10
+ */
+#define gtk_widget_class_bind_template_child_internal(widget_class, TypeName, member_name) \
+  gtk_widget_class_bind_template_child_full (widget_class, \
+                                             #member_name, \
+                                             TRUE, \
+                                             G_STRUCT_OFFSET (TypeName, member_name))
+
+/**
+ * gtk_widget_class_bind_template_child_private:
+ * @widget_class: a #GtkWidgetClass
+ * @TypeName: the type name of this widget
+ * @member_name: name of the instance private member in the private struct for @data_type
+ *
+ * Binds a child widget defined in a template to the @widget_class.
+ *
+ * This macro is a convenience wrapper around the
+ * gtk_widget_class_bind_template_child_full() function.
+ *
+ * This macro will use the offset of the @member_name inside the @TypeName
+ * private data structure (it uses G_PRIVATE_OFFSET(), so the private struct
+ * must be added with G_ADD_PRIVATE()).
+ *
+ * Since: 3.10
+ */
+#define gtk_widget_class_bind_template_child_private(widget_class, TypeName, member_name) \
+  gtk_widget_class_bind_template_child_full (widget_class, \
+                                             #member_name, \
+                                             FALSE, \
+                                             G_PRIVATE_OFFSET (TypeName, member_name))
+
+/**
+ * gtk_widget_class_bind_template_child_internal_private:
+ * @widget_class: a #GtkWidgetClass
+ * @TypeName: the type name, in CamelCase
+ * @member_name: name of the instance private member on the private struct for @data_type
+ *
+ * Binds a child widget defined in a template to the @widget_class, and
+ * also makes it available as an internal child in GtkBuilder, under the
+ * name @member_name.
+ *
+ * This macro is a convenience wrapper around the
+ * gtk_widget_class_bind_template_child_full() function.
+ *
+ * This macro will use the offset of the @member_name inside the @TypeName
+ * private data structure.
+ *
+ * Since: 3.10
+ */
+#define gtk_widget_class_bind_template_child_internal_private(widget_class, TypeName, member_name) \
+  gtk_widget_class_bind_template_child_full (widget_class, \
+                                             #member_name, \
+                                             TRUE, \
+                                             G_PRIVATE_OFFSET (TypeName, member_name))
+
+GDK_AVAILABLE_IN_3_10
+void    gtk_widget_init_template                        (GtkWidget             *widget);
+GDK_AVAILABLE_IN_3_10
+GObject *gtk_widget_get_template_child                  (GtkWidget             *widget,
+						         GType                  widget_type,
+						         const gchar           *name);
+GDK_AVAILABLE_IN_3_10
+void    gtk_widget_class_set_template                   (GtkWidgetClass        *widget_class,
+						         GBytes                *template_bytes);
+GDK_AVAILABLE_IN_3_10
+void    gtk_widget_class_set_template_from_resource     (GtkWidgetClass        *widget_class,
+						         const gchar           *resource_name);
+GDK_AVAILABLE_IN_3_10
+void    gtk_widget_class_bind_template_callback_full    (GtkWidgetClass        *widget_class,
+						         const gchar           *callback_name,
+						         GCallback              callback_symbol);
+GDK_AVAILABLE_IN_3_10
+void    gtk_widget_class_set_connect_func               (GtkWidgetClass        *widget_class,
+						         GtkBuilderConnectFunc  connect_func,
+						         gpointer               connect_data,
+						         GDestroyNotify         connect_data_destroy);
+GDK_AVAILABLE_IN_3_10
+void    gtk_widget_class_bind_template_child_full       (GtkWidgetClass        *widget_class,
+						         const gchar           *name,
+						         gboolean               internal_child,
+						         gssize                 struct_offset);
+
 G_END_DECLS
 
 #endif /* __GTK_WIDGET_H__ */
